/*
 *  Miscellaneous functions used by the cdecl99 test suite.
 *  Copyright © 2011 Nick Bowler
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <config.h>
#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <cdecl.h>
#include "test.h"

void *realloc_nofail(void *ptr, size_t size)
{
	void *p;
	
	p = realloc(ptr, size);
	if (!p) {
		perror("failed to allocate memory");
		abort();
	}

	return p;
}

void *malloc_nofail(size_t size)
{
	return realloc_nofail(NULL, size);
}

void test_print_decl(struct cdecl *decl)
{
	static size_t bufsz;
	static char *buf;
	size_t rc;

retry:
	rc = cdecl_declare(buf, bufsz, decl);
	if (rc >= bufsz) {
		bufsz = rc + 1;
		buf = realloc_nofail(buf, bufsz);
		goto retry;
	}

	printf("%s\n", buf);
}

bool strict_strtoul(unsigned long *val, const char *str, int base)
{
	char *end;

	errno = 0;
	*val = strtoul(str, &end, base);
	if (errno != 0 || *end != 0)
		return false;

	return true;
}

void test_print_version(const char *program)
{
	printf("%s (%s) %s\n", program, PACKAGE_NAME, PACKAGE_VERSION);
	puts("Copyright (C) 2011 Nick Bowler.");
	puts("License GPLv3+: GNU GPL version 3 or later <http://gnu.org/licenses/gpl.html>.");
	puts("This is free software: you are free to change and redistribute it.");
	puts("There is NO WARRANTY, to the extent permitted by law.");
}
